// XpaDemoDataDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XpaDemo.h"

#include "XpipeAPI.h"

#include "XpaDemoDataDlg.h"
#include "XpaDemoClearReqDlg.h"		// Clear Request dialog ...

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// Define this to emulate transmit ready notifications. Currently i've got
// a problems with tx interrupts so what i do is when i fail to transmit 
// with return code RC_NOT_READY i post WMXPA_TXREADY message to myself. 
// In the final version this message will be sent by the API.
//#define XPADEMO_EMULATE_TXREADY

/////////////////////////////////////////////////////////////////////////////
// Prototype for display clear array - see XpaDemoDlg.cpp for details
extern BOOL	gDisplayClearReq[];


/////////////////////////////////////////////////////////////////////////////
// CXpaDemoDataDlg dialog


CXpaDemoDataDlg::CXpaDemoDataDlg(int Lcn, CWnd* pParent /*=NULL*/)
	: CDialog(CXpaDemoDataDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CXpaDemoDataDlg)
	m_txLen = 128;
	m_txNum = 100;
	m_rxCorrString = _T("");
	m_rxOutOfSeqString = _T("");
	m_rxFramesString = _T("");
	//}}AFX_DATA_INIT

	// remember the logical channel number
	m_lcn = Lcn;

	// initialize the transmit flag
	m_txing = FALSE;
	// initialize the tx sequence number
	m_currentTxSeq = 0;
	// initialize the rx sequence number
	m_currentRxSeq = 0;
	// initialize the number of received frames
	m_rxFrames = 0;
	// initialize the number of out-of-sequence frames
	m_rxOutOfSeq = 0;
	// initialize the number of corrupted frames
	m_rxCorr = 0;

	// display the dialog ...
	Create(IDD_XPADEMO_DATA_DIALOG, pParent);

	// set dialog caption ...
	{
		char	szCaption[MAX_PATH];

		sprintf(szCaption, "XpaDemo - Data Transfer on logical channel %d", m_lcn);
		SetWindowText( szCaption );
	}

	// register this window with XpipeAPI
	xpaRegisterWindow( m_hWnd, 1, m_lcn );

	// indicate that we're going to take care of Clear Requests
	gDisplayClearReq[m_lcn] = FALSE;

	// prepare the data buffer for transmissions
	{
		int	iIndex;
		for (iIndex = 0; iIndex < XPA_CBLOCK_MDATALEN; iIndex++) {
			m_txDataBuffer[iIndex] = (BYTE)iIndex;
		}
	}
}


void CXpaDemoDataDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CXpaDemoDataDlg)
	DDX_Control(pDX, IDC_OUTDATA, m_outDataListBox);
	DDX_Control(pDX, IDC_INDATA, m_inDataListBox);
	DDX_Control(pDX, IDC_TXSTART, m_txStartButton);
	DDX_Text(pDX, IDC_TXLEN, m_txLen);
	DDV_MinMaxInt(pDX, m_txLen, 0, 1024);
	DDX_Text(pDX, IDC_TXNUM, m_txNum);
	DDX_Text(pDX, IDC_RXCORR, m_rxCorrString);
	DDX_Text(pDX, IDC_RXOUTSEQ, m_rxOutOfSeqString);
	DDX_Text(pDX, IDC_RXFRAMES, m_rxFramesString);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CXpaDemoDataDlg, CDialog)
	//{{AFX_MSG_MAP(CXpaDemoDataDlg)
	ON_BN_CLICKED(IDC_DATACLOSE, OnCloseClicked)
	ON_BN_CLICKED(IDC_TXSTART, OnTxStartClicked)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMXPA_RXDATA, OnRxData)			// Receive data
	ON_MESSAGE(WMXPA_TXREADY, OnTxReady)		// Transmit ready
	ON_MESSAGE(WMXPA_ASY_CLEARREQ, OnClearReq)	// Clear Request
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXpaDemoDataDlg message handlers

//
// "Close" button clicked
//
void CXpaDemoDataDlg::OnCloseClicked() 
{
	// tell the main window that it will have to take care of Clear Requests 
	gDisplayClearReq[m_lcn] = TRUE;

	// deregister this window from XpipeAPI - note that the function always works
	// even if the window wasn't actually registered.
	xpaDeregisterWindow( m_hWnd, 1 );
	
	// destroy the dialog 
	DestroyWindow();

	// since we created the dialog on heap - we need to delete it as well
	delete this;
}

//
// "Start/Stop Transmit" button clicked - it changes its meaning depending on the
// current state - whether we're transmitting or not 
//
void CXpaDemoDataDlg::OnTxStartClicked() 
{
	if (m_txing) {
		// stop transmit ....

		// add the separation line to the output listbox
		m_outDataListBox.SetCurSel(m_outDataListBox.AddString("<->\t<->\t<->\t<->\t<->\t<->"));
		// reset the transmit flag
		m_txing = FALSE;
		// set the button capture back to "Start Transmit"
		m_txStartButton.SetWindowText("Start Transmit");
	} else {
		// start transmit ....

		// get dialog data - we will be interested in tx length 
		UpdateData( TRUE );

		// set the transmit flag
		m_txing = TRUE;
		// set the button capture to "Stop Transmit"
		m_txStartButton.SetWindowText("Stop Transmit");
		// reset the number of transmitted frames
		m_framestxed = 0;
		// start transmitting ...
		TransmitData();
	}
}

//
//  WMXPA_RXDATA handler
//
afx_msg LRESULT CXpaDemoDataDlg::OnRxData(WPARAM wParam, LPARAM lParam)
{
	char			szDataString[MAX_PATH];				// string for list bx
	XPA_MSG_DATA   *pMsgData = (XPA_MSG_DATA *)lParam;  // msg. data
	int				iIndex, iBytesToPrint;		

	// format the string to display: DataLengh First 15 bytes ...
	sprintf( szDataString, "%d\t", pMsgData->buffer_length );
	iBytesToPrint = (pMsgData->buffer_length <= 15) ? pMsgData->buffer_length : 15;
	for (iIndex = 0; iIndex < iBytesToPrint; iIndex++) {
		sprintf(szDataString+strlen(szDataString), "%02X ", pMsgData->data[iIndex]);
	}
	sprintf(szDataString+strlen(szDataString), "...");

	// add the string to the list box and select it 
	m_inDataListBox.SetCurSel(m_inDataListBox.AddString(szDataString));

	// increment the number of rx frames
	m_rxFrames++;

	// check the sequence number .. if its out of sequence - increment the counter
	if (m_currentRxSeq != pMsgData->data[0]) {
		// increment the counter
		m_rxOutOfSeq++;
		// set a new value for the current receive sequence number
		m_currentRxSeq = pMsgData->data[0];
	}
	// increment the sequence number for the next time ...
	m_currentRxSeq++;

	// check the data integrity within a frame
	for (iIndex = 1; iIndex < (pMsgData->buffer_length - 1); iIndex++) {
		if ( (BYTE)(pMsgData->data[iIndex]+1) != (BYTE)pMsgData->data[iIndex+1] ) {
			// frame appears to be corrupted 
			m_rxCorr++;
			break;
		}
	}

	// update dialog data
	m_rxFramesString.Format("%d", m_rxFrames);
	m_rxOutOfSeqString.Format("%d", m_rxOutOfSeq);
	m_rxCorrString.Format("%d", m_rxCorr);
	UpdateData( FALSE );

	// notify API we've proccessed the message
	xpaMsgComplete( (XPA_MSG_DATA *)lParam );

	// done
	return 0;
}

//
//  WMXPA_TXREADY handler
//
afx_msg LRESULT CXpaDemoDataDlg::OnTxReady(WPARAM wParam, LPARAM lParam)
{
	// notify the API that we've processed the message 
	xpaMsgComplete( (XPA_MSG_DATA *)lParam );
	// keep on transmitting ... 
	TransmitData();
	return 0;
}

//
//  WMXPA_ASY_CLEARREQ handler
//
afx_msg LRESULT CXpaDemoDataDlg::OnClearReq(WPARAM wParam, LPARAM lParam)
{
	// first off - stop transmitting we we are ...
	if (m_txing) {
		OnTxStartClicked();
	}

	// display the Clear Request dialog box ... note that the message acknowledgement
	// will be performed from there since we will need to have message data available 
	CXpaDemoClearReqDlg *dlg = new CXpaDemoClearReqDlg( (int)wParam, (XPA_MSG_DATA *)lParam, this );

	return 0;
}

//
// Default message handler - we need it to handle correctly all unhandled 
// WMXPA_ messages ...
//
LRESULT CXpaDemoDataDlg::DefWindowProc( UINT message, WPARAM wParam, LPARAM lParam )
{
	if ((message >= WMXPA_ASY_INCALL) && (message <= WMXPA_RXDATA)) {
		// notify the API that we've processed the message 
		xpaMsgComplete( (XPA_MSG_DATA *)lParam );
	}
	// call the parent class 
	return CDialog::DefWindowProc( message, wParam, lParam );
}

//
// This member function is the one that actually takes care of transmission of 
// data. It pushes the data until a non-zero return code has been received from
// xpaWriteData(). It checks the return code and if it's other than RC_NOT_READY
// it will display an appropriate message giving the user an option of either
// continue transmitting or terminating ... If the return code is RC_NOT_READY
// we just exit, because later on we will receive WMXPA_TXREADY message and will
// continue from there ...
//
void  CXpaDemoDataDlg::TransmitData(void)
{
	char	szDataString[MAX_PATH];			// for list box 
	int		iBytesToPrint, iIndex;			// for list box
	int		iRVal;							// return value from xpaWriteData
		
	// Number of bytes to print in the list box .... and data to display
	iBytesToPrint = (m_txLen <= 15) ? m_txLen : 15;

	// Start sending ...
	while ( m_txing ) {

		// set the first byte of the frame to the current tx sequence number
		m_txDataBuffer[0] = m_currentTxSeq;

		// try to send ...
		if (0 == ( iRVal = xpaWriteData( 1, m_lcn, m_txDataBuffer, m_txLen, 0 ))) {
			
			// send successfull ...

			// format the string to display ...
			sprintf( szDataString, "%d\t", m_txLen );
			for (iIndex = 0; iIndex < iBytesToPrint; iIndex++) {
				sprintf(szDataString+strlen(szDataString), "%02X ", m_txDataBuffer[iIndex]);
			}
			sprintf(szDataString+strlen(szDataString), "...");

			// add the string to the list box and select it 
			m_outDataListBox.SetCurSel(m_outDataListBox.AddString(szDataString));

			// increment the transmit sequence number
			m_currentTxSeq++;

			// increment the number of tx'd frames - if it reaches m_txNum - exit
			if (++m_framestxed == m_txNum) OnTxStartClicked();

		} else {

			// send failed ... take a look on return code
			if (iRVal == RC_NOT_READY) {
				// ok, in this case we will exit now, and later on we will
				// get the WMXPA_TXREADY message which will continue to 
				// transmit ...
#ifdef XPADEMO_EMULATE_TXREADY
				// ok, if we're emulating WMXPA_TXREADY we post this message to
				// ourselves instead of API
				PostMessage( WMXPA_TXREADY, (WPARAM)m_lcn, (LPARAM)NULL );
#endif
				return;
			}

			// some other return code - display message box
			char	szOutputString[MAX_PATH];

			sprintf(szOutputString, "Failed to transmit data frame. Return code 0x%02X, last error %d\nWould you like to continue sending data ?", iRVal, xpaGetLastError());
			if (IDNO == AfxMessageBox( szOutputString, MB_YESNO )) {
				// user decided to terminate the send ... this function
				// will reset m_txing flag and we will terminate ...
				OnTxStartClicked();
			}
		}
	}
	// done ...
}
