/*
 * ConsoleRx.c   - console application for demonstration on how to use event
 *				   mechanism with XpipeAPI.
 *
 *				   The application receives data on specified logical channel. 
 *				   Calls establishing if needed can be done from XpaDemo.exe
 *
 */

#include "ConsoleRx.h"

/* i'll make the main XPA event and the event data global just for		*/
/* convinience	it doesn't have to be global of course ....				*/
HANDLE				hXpaEvent = NULL;
XPA_EVENT_DATA		gXpaEventData = { 0, 0, NULL };

/* local functions prototypes */
BOOL WINAPI TerminateHandler(DWORD);

/*
 * main thread - takes care of all the initialization stuff and creates receive thread
 *
 *				 I know there's no really point to have a separate thread for receives
 *				 but i originally wanted to do a little bit more than just receives but then
 *				 i didnt have enough
 *
 */
main(int argc, char *argv[])
{
	XPA_X25CFG		X25Cfg;				/* X.25 configuration */
	int				iRVal;				/* return value from various XPA commands */
	int				iLcn;				/* logical channel */
	BOOL			IsDTM, TxWinOpen;	/* for reading channel status */
	int				InPackets;			/* for reading channel status */
	int				iIndex;				/* index */
	int				iDisplayLen;		/* length of data to display */
	UCHAR			currentRxSeq=0;		/* current Rx seq number */

	/* say something ... */
	printf("XpipeAPI console receiver. Copyright 1998 Sangoma Technologies Inc.\n");
	
	/* register Ctrl+C handler - coz from we want a clean termination */
	if (!SetConsoleCtrlHandler(TerminateHandler, TRUE)) {
		printf("ERROR : Unable to register terminate handler( %d ).\nProcess terminated.\n", GetLastError());
		exit(-1);
	}

	/* open XpipeAPI library - use the default device */
	if (!xpaOpen(NULL)) {
		printf("ERROR : Unable to initialize XpipeAPI library ( %d ).\nProcess terminated.\n", xpaGetLastError());
		exit(-1);
	}

	/* read X.25 configuration - we are interested in available logical */
	/* channels numbers													*/
	if (0 != (iRVal=xpaReadConfiguration( 1, &X25Cfg ))) {
		/* failure */
		printf("ERROR : Unable to read X.25 configuration. Return value 0x%02X, last error %d.\nProcess terminated.\n", iRVal, xpaGetLastError());
		xpaClose(1);
		exit(-1);
	}
	
	/* go through the configured channels checking the state and display the list */
	printf("Logical channels ready to transfer data : ");
	for ( iLcn = X25Cfg.PVClow; iLcn <= X25Cfg.PVChigh; iLcn++ ) {
		xpaGetLcnStatus( 1, iLcn, &IsDTM, &TxWinOpen, &InPackets );
		if (IsDTM) printf("%d(pvc) ", iLcn);
	}
	for ( iLcn = X25Cfg.SVClow; iLcn <= X25Cfg.SVChigh; iLcn++ ) {
		xpaGetLcnStatus( 1, iLcn, &IsDTM, &TxWinOpen, &InPackets );
		if (IsDTM) printf("%d(svc) ", iLcn);
	}
	printf("\nPlease, note that some of the above channels may be in use by Xpipe508 !!!\n");

	/* read the logical channel number */
	printf("\nPlease enter the logical channel number for data transfer or 0 to terminate : \n");
	scanf("%d", &iLcn);
	/* check for terminate */
	if (0 == iLcn) {
		printf("Process terminated by user request.\n");
		xpaClose(1);
		exit(0);
	}
	
	/* seem that we've got go-ahead for transfers ... */

	/* 
	 * create an event for receive notifications. 
	 */
	hXpaEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (NULL == hXpaEvent) {
		printf("ERROR : Unable to create an event object ( %d ).\nProcess terminated.\n", GetLastError());
		xpaClose(1);
		exit(-1);
	}

	/* ok, now register event with XpipeAPI */
	if (!xpaRegisterEvent( hXpaEvent, 1, iLcn, &gXpaEventData )) {
		/* failure */
		printf("ERROR : Unable to register an event with XpipeAPI (%d).\nProcess terminated.\n", xpaGetLastError());
		xpaClose(1);
		exit(-1);
		/* note that the receive thread is waiting on hXpaEvent and at this point will */
		/* be awaken with WAIT_ABANDONED and therefore terminate ...				   */
	}

	/* 
	 * start up the receive loop .... the loop can be terminated at any time since all the cleanup
	 * is done from TerminateHandler() 
	 */
	printf("Starting receive ...\n");

	while(1) {
		if (WAIT_ABANDONED == WaitForSingleObject(hXpaEvent, INFINITE)) {
			/* gotta terminate */
			break;
		}
		/* check the event type and logical channle number */
		if ((gXpaEventData.EventType == EVXPA_RXDATA) && (gXpaEventData.Lcn == iLcn)) {
			/* looks like this one is for us !!! */
			printf("Received %d bytes, Lcn %d : ", gXpaEventData.pMsgData->buffer_length, iLcn);
			iDisplayLen = (gXpaEventData.pMsgData->buffer_length < 5) ? gXpaEventData.pMsgData->buffer_length : 5;
			for (iIndex=0; iIndex < iDisplayLen; iIndex++) {
				printf("%02X ", gXpaEventData.pMsgData->data[iIndex]);
			}

			printf("... ");

			/* check the sequence number ... say something if its out of sequence */
			if (currentRxSeq != gXpaEventData.pMsgData->data[0]) {
				printf("OUT of sequence !!! ");
				/* set the sequence number for the next time */
				currentRxSeq = gXpaEventData.pMsgData->data[0];
			} else {
				printf("OK ");
			}
			/* increment the sequence number for the next time */
			currentRxSeq++;

			/* check the data integrity within a frame */
			for (iIndex = 1; iIndex < (gXpaEventData.pMsgData->buffer_length - 1); iIndex++) {
				if ( (UCHAR)(gXpaEventData.pMsgData->data[iIndex]+1) != (BYTE)gXpaEventData.pMsgData->data[iIndex+1] ) {
					/* frame appears to be corrupted */
					break;
				}
			}
			if (iIndex < (gXpaEventData.pMsgData->buffer_length - 1)) {
				printf("Frame corrupted !!!\n");
			} else {
				printf("OK\n");
			}
			/* done with this frame */
		}
		/* acknoledge the event anyways */
		xpaEventComplete(hXpaEvent);

		/* go on ... Sleep(0) will cause to reschedule the thread therefore giving a main thread */
		/* chance to run sooner	which is good in case it wasn't the receive event				 */
		Sleep(0); 
	}

	/* almost never reached */
	return 0;
}


/*
 * TerminateHandler() - this handler is called by the system whenever user tries to terminate
 *						the process with Ctrl+C, Ctrl+Break or closes the console window.
 *						We would like to perform a clean-up here especially considering the 
 *						fact that this is the only way to terminate the process.
 */
BOOL WINAPI 
TerminateHandler(DWORD dwCtrlType)
{
	/* deregister the event from XpipeAPI */
	xpaDeregisterEvent( hXpaEvent, 1 );
	/* we don't need the event anymore ... */
	CloseHandle( hXpaEvent );
	/* deregister from XpipeAPI */
	xpaClose(1);
	/* say something */
	printf("\nProcess terminated by user request.\n");
	/* return FALSE so the system will call the dafult handler which will terminate the process */
	return FALSE;
}
