/*
 * XpipeAPI.h - user include file for XpipeAPI.dll 
 *
 *
 *	Author		: Anthony Belov, abelov@sangoma.com, Sangoma Technologies Inc.
 *
 *	Date		: October, 1998
 *
 *	Copyright	: Sangoma Technologies Inc, 1998
 *
 * July 23,		2002	David Rokhvarg	v2.0.0.0
 *						Added new return codes for different internal
 *						errors instead of one and only XPA_EINT.
 *						Added a separate socket for acknowleging Rx data frames.
 *
 * November 11, 2002	David Rokhvarg	v2.0.0.1
 *						Checking for buffer size instead of relying on exception
 *						handling when CopyMemory() is used. Because CopyMemory()
 *						may NOT throw an exeption when destination buffer is too small.
 *
 */
#ifndef _XPIPEAPI_H_
#define _XPIPEAPI_H_

/*
 * -------------------------------* Constants  *------------------------------
 */

/* --- xpa error codes returned with xpaGetLastError() --- */

#define XPA_NOERROR			0	/* no error */
#define XPA_ENOMEM			1	/* not enough memory to complete operation */
#define XPA_ENODEVCON		2	/* couldn't establish connection to device */
#define XPA_EINVDEV			3	/* invalid device name/device not found */
#define XPA_EADDR			4	/* invalid address */
#define XPA_EINT			5	/* internal error has occurred */
#define XPA_EBOARD_LOCKED	6	/* other thread has blocked the board */
#define XPA_EBOARD_TIMEOUT	7	/* timeout has occurred while executing board cmd */
								/*
								   If returned on the first call to the card most likely
								the UDP call is not reaching XPIPE driver. Possible reasons
								for that could be:
								1. no IP address was assigned to the card.
								2. driver was not restarted after IP address was assigned.
								3. the IP addr and Net Mask supplied for the XPIPE
								   do not allow to have at least two hosts on the subnet and
								   no remote IP address can be construted.
								   If returned after a number of successfull calls to the card,
								the code on	the card is	not running or card is too busy.
								*/
#define XPA_EINV_ARGS		8	/* invalid argument(s) supplied */
#define XPA_ESOCKET_SEND	9	/* send() on cmd socket failed */
#define XPA_ECMD_NEXT_ID	10	/* Id mismatch on return from cmd */
#define XPA_ECMD_REQUESTREPLY			11	/* Request/Replay mismatch on return from cmd */
#define XPA_EMESSAGE_HANDLING_THREAD	12	/* failed to create a message handling thread */
#define XPA_EEVENT_HANDLING_THREAD		13	/* failed to create a event handling thread */
#define XPA_EASYNC_SOCK_BIND			14	/* failed to bind Async data socket */
#define XPA_EASYNC_NOTIFY				15	/* failed to set asynchronous notification */


/* ---- X.25 board level command codes ---- */

#define XPA_LINK_STATUS						0x06
#define XPA_READ_HDLC_STATISTICS			0x07
#define XPA_FLUSH_HDLC_STATISTICS			0x08
#define XPA_READ_COMMS_ERR_STATS			0x09
#define XPA_FLUSH_COMMS_ERR_STATS			0x0a
#define XPA_SET_GLOBAL_VARIABLES			0x0b
#define XPA_READ_MODEM_STATUS				0x0c
#define XPA_OPERATE_DATALINE_MONITOR		0x14
#define XPA_READ_CODE_VERSION				0x15
#define XPA_READ_TRACE_DATA					0x16
#define XPA_X25_READ						0x22
#define XPA_X25_WRITE						0x23
#define XPA_PLACE_CALL						0x30
#define XPA_ACCEPT_CALL						0x31
#define XPA_CLEAR_CALL						0x32
#define XPA_CONFIRM_CLEAR					0x33
#define XPA_RESET_LOGICAL_CHANNEL			0x34
#define XPA_CONFIRM_RESET					0x35
#define XPA_RESTART							0x36
#define XPA_CONFIRM_RESTART					0x37
#define XPA_INTERRUPT						0x38
#define XPA_CONFIRM_INTERRUPT				0x39
#define XPA_REGISTRATION_REQUEST			0x3a
#define XPA_REGISTRATION_CONFIRMATION		0x3b
#define XPA_IS_DATA_AVAILABLE				0x40
#define XPA_INCOMING_CALL_CONTROL			0x41
#define XPA_CONFIGURE_PVC					0x42
#define XPA_READ_ACTIVE_CHANNELS			0x43
#define XPA_READ_CHANNEL_CONFIGURATION		0x44
#define XPA_FLUSH_X25_DATA_BUFFERS			0x45
#define XPA_READ_X25_HISTORY_TABLE			0x46
#define XPA_X25_HISTORY_TABLE_CONTROL		0x47
#define XPA_READ_X25_STATISTICS				0x49
#define XPA_FLUSH_X25_STATISTICS			0x4a
#define XPA_READ_HDLC_X25_CONFIGURATION		0x50
#define XPA_SET_HDLC_X25_CONFIGURATION		0x51

/* ---- X.25 board level return codes ---- */

#define RC_CMD_OK							0x00
#define RC_FAILED							0x01
#define RC_LINK_NOT_IN_ABM					0x02
#define RC_LINK_CLOSED						0x03
#define RC_INVALID_BUFFLEN					0x04
#define RC_INVALID_COMMAND					0x05
#define RC_UNNUMBERED_FRAME_RECEIVED		0x06
#define RC_LINK_IN_FRAME_REJECT_MODE		0x07
#define RC_MODEM_FAILURE					0x08
#define RC_RETRY_LIMIT_EXCEEDED				0x09
#define RC_INVALID_LCN						0x30
#define RC_INVALID_STATE					0x31
#define RC_INVALID_DATA_LENGTH				0x32
#define RC_NOT_READY						0x33
#define RC_NETWORK_OUT_OF_ORDER				0x34
#define RC_CHANNEL_IN_USE					0x35
#define RC_INVALID_ARGUMENT					0x3A
#define RC_ASYNCH_PACKET_RECEIVED			0x40
#define RC_PROTOCOL_VIOLATION				0x41
#define RC_PACKET_TIMEOUT					0x42
#define RC_ASYNCH_PACKET_RETRY_OVRFLW		0x43
/* command dependant resutls */
#define RC_LINK_DISCONNECTED				0x00	/* GET_LINK_STATUS */
#define RC_LINK_IN_ABM						0x01	/* GET_LINK_STATUS */
#define RC_NO_DATA_AVAILABLE				0x01	/* HDLC_READ/READ_SCOPE_DATA*/
#define RC_SCOPE_IN_USE						0x02	/* READ_SCOPE_DATA */
#define RC_LINK_ALREADY_OPEN				0x01	/* OPEN_LINK */
#define RC_LINK_ALREADY_DISCONNECTED		0x02	/* LINK_DISCONNECT */
#define RC_LINK_ALREADY_CLOSED				0x03	/* CLOSE_LINK */
#define RC_INVALID_CONFIGURATION			0x35	/* REGISTR_RQST/CONFRM */

/* ---- maximum valid logical channel number ---- */
#define XPA_MAX_LCN							4095

////////////////////////////////////////////////////////////////////////////////////

/*
 * ---------------------------------*  Macros  *------------------------------
 */

/*
 * -------------------------* Data structures and types  *--------------------
 */

/* X.25 control block - for use with xpaDoCommand() function call	*/
/* Its packed on 1 byte boundary because it will go directly to the */
/* board for execution.												*/
#define XPA_CBLOCK_MDATALEN			1027	/* data buffer length	*/

#pragma pack(1)
typedef struct {
		BYTE	command;					/* command code */
		WORD	buffer_length;				/* buffer length */
		BYTE	return_code;				/* return code */
		BYTE	reserved_HDLC;				/* */
		WORD	logical_channel;			/* logical channel for command */
		BYTE	Q_D_M_bits;					/* Q/D/M bits */
		BYTE	cause;						/* cause byte */
		BYTE	diagnostic;					/* diagnostic byte */
		BYTE	X25_packet_type;			/* X.25 packet type byte */
		DWORD	time_stamp;					/* time stamp */
		BYTE	data[XPA_CBLOCK_MDATALEN];	/* data buffer */
} XPA_CBLOCK, *PXPA_CBLOCK;
#pragma pack()

/* X.25 configuration structure returned from xpaReadConfiguration() call */
#pragma pack(1)
typedef struct {
		BYTE	LineSpeedIndex;				/* line speed index */
		BYTE	T1;							/* T1 value */	
		BYTE	T2;							/* T2 value */
		BYTE	N2;							/* N2 value */
		WORD	MaxIFrameSize;				/* Maximum iframe size */
		BYTE	HdlcWindow;					/* HDLC level window */
		BYTE	T4;							/* T4 value */
		BYTE	AutoModem;					/* 'auto_modem_fixup' parameter */
		BYTE	AutoHdlc;					/* 'auto_hdlc' parameter */
		BYTE	HdlcConfig;					/* HDLC configuration options */
		BYTE	DTE;						/* Station HDLC level config */
		BYTE	X25Window;					/* X.25 packet window */
		WORD	DefaultDataSize;			/* Default X.25 data field size */
		WORD	MaxDataSize;				/* Max X.25 data field size */
		WORD	PVClow;						/* The lowest PVC */
		WORD	PVChigh;					/* The highest PVC */
		WORD	inSVClow;					/* The lowest incoming SVC */
		WORD	inSVChigh;					/* The highest incoming SVC */
		WORD	SVClow;						/* The lowest two-way SVC */
		WORD	SVChigh;					/* The highest tow-way SVC */
		WORD	outSVClow;					/* The lowest outgoing SVC */
		WORD	outSVChigh;					/* The highest outgoing SVC */
		WORD	X25Config;					/* X.25 configuration options */
		BYTE	X25response;				/* X.25 response options */
		WORD	GenFacil1;					/* General facilities 1 */
		WORD	GenFacil2;					/* General facilities 2 */
		WORD	CCITTFacil;					/* CCITT facilities */
		WORD	NonX25Facil;				/* Non-X.25 facilities */
		WORD	Compat;						/* Compatibility version */
		BYTE	T10;						/* T10/T20 value */
		BYTE	T11;						/* T11/T21 value */
		BYTE	T12;						/* T12/T22 value */
		BYTE	T13;						/* T13/T23 value */
		BYTE	T16;						/* T16/T26 value */
		BYTE	T28;						/* T28 value */
		BYTE	R10;						/* R10/R20 value */
		BYTE	R12;						/* R12/R22 value */
		BYTE	R13;						/* R13/R23 value */
} XPA_X25CFG, *PXPA_X25CFG;
#pragma pack()

/* Data structure pointed by the message parameter passed with WMXPA_ messages,  */
/* which indicate asynchronous X.25 events										 */
#pragma pack(1)
typedef struct {
		WORD	buffer_length;		/* length of data associated */
		BYTE	Q_D_M_bits;			/* Q/D/M bits */
		BYTE	cause;				/* cause byte */
		BYTE	diagnostic;			/* diagnostic byte */
		WORD	reserved;			/* */
		BYTE	data[1];			/* first byte of data buffer */
} XPA_MSG_DATA, *PXPA_MSG_DATA;
#pragma pack()

/* Data strucutre used for event notifications - a container allocated by user	*/
/* where the API will store the data for asynchronous events.					*/
typedef struct {
		UINT			EventType;	/* type of asynchronous event - see below */
		int				Lcn;		/* logical channel number for this event */
		XPA_MSG_DATA	*pMsgData;	/* pointer to the message data structure */
									/* used to hold all the rest of the info */
									/* for the asynchronous event			 */
} XPA_EVENT_DATA, *PXPA_EVENT_DATA;


/*
 * -----------------------------* Windows Messages   *-------------------------
 */

/* 
 * We are using windows messages starting from (WM_APP+100) to notify windows 
 * registered with xpaRegisterWindow() of various interesting events such as 
 * reception of X.25 asynchronous packets, X.25 protocol violations, timeouts 
 * and retry overflows for the previously issued X.25 commands, availability of
 * transmit buffers for certain logical channel and the reception of the data 
 * packets from the network.
 * Most of the messages conform to the following format ( differences are 
 * outlined ) :
 *
 *		WPARAM wParam - logical channel number on which the event of interest has
 *						occurred.
 *
 *		LPARAM lParam - pointer to the XPA_MSG_DATA structure allocated by the API
 *						containing the packet parameters and data.
 *
 *	Important Notes:
 *
 *			These messages will be posted to the windows registered with xpaOpen()
 *			call whenever an asynchronous X.25 packet has been received from the
 *			network. Once the application has finished processing the message and
 *			does not require XPA_MSG_DATA associated with it anymore it should call
 *			xpaMsgComplete() supplying the pointer to the XPA_MSG_DATA so the API 
 *			will be able to release the memory allocated for the strucutre. What it
 *			means is that you should always call xpaMsgComplete((XPA_MSG_DATA *)lParam)
 *			before returning from your message handler. 
 *
 *			Please also note that the API keeps track of number of windows it posted
 *			the message to and will not release the memory until all of the windows 
 *			registrered with it have called xpaMsgComplete().
 *
 *			For the safety sake the amount of memory allocated for the outstanding
 *			message data structures is limited to about 1 Meg. If the API gets to the
 *			point where it either can not allocate the memory for the message structure
 *			or the amount of the outstanding memory is close to the limit it disables
 *			all the interrupts from the board which of course will interfere with a 
 *			regular IP traffic as well ... 
 *
 *			The amount of memory allocate for each message data sturcutre is the 
 *			structure size plus the number of bytes specified in buffer_length field.
 *			Accessing memory past that will most likely cause an exception.
 *
 *			Please note that responses to Clear Request, Reset Request, Restart Request 
 *			and Interrupt packets are issued automatically by the board, therefore these
 *			messages sent to you only for notification purposes, i.e. you do not need to
 *			use any of xpaConfirmClear(), xpaConfirmReset() and xpaConfirmInterrupt() to
 *			respond to these messages. 
 *
 *			Also please keep in mind :
 *
 *				on reception of WMXPA_ASY_CLEARREQ the logical channel it was issued for
 *				is gone - the board confirms clear requests automatically, so for any 
 *				subsequent operation on this channel you will get RC_INVALID_LCN. 
 *
 *				on reception of WMXPA_ASY_RESTARTREQ every SVC is reset, therefore all the
 *				SVC logical channels are gone - the board confirms restart requests 
 *				automatically. 
 *
 */

/* --- Asynchronous X.25 packets reception messages --- */

#define WMXPA_ASY_INCALL		(WM_APP+100)	/* Call Request/Incoming call */
#define WMXPA_ASY_CONNECTED		(WM_APP+101)	/* Call Accepted/Connected */

#define WMXPA_ASY_CLEARREQ		(WM_APP+102)	/* Clear Request/Indication */
#define WMXPA_ASY_CLEARCONF		(WM_APP+103)	/* Clear Confirmation */

#define WMXPA_ASY_RESETREQ		(WM_APP+104)	/* Reset Request/Indication */
#define WMXPA_ASY_RESETCONF		(WM_APP+105)	/* Reset Confirmation */

#define WMXPA_ASY_RESTARTREQ	(WM_APP+106)	/* Restart Request/Indication */
#define WMXPA_ASY_RESTARTCONF	(WM_APP+107)	/* Restart Confirmation */

#define WMXPA_ASY_INTERREQ		(WM_APP+108)	/* Interrupt */
#define WMXPA_ASY_INTERCONF		(WM_APP+109)	/* Interrupt Confirmation */

#define WMXPA_ASY_DTEREQ		(WM_APP+110)	/* DTE Registration Request */
#define WMXPA_ASY_DTECONF		(WM_APP+111)	/* DTE Registration Confirmation */

#define WMXPA_ASY_DIAG			(WM_APP+112)	/* Diagnostic message */


/* --- Protocol violation messages - a protocol violation occurred and an X.25  --- */
/* --- asynchronous packet has been automatically issued to notify the remote   --- */
/* --- device of this violation. The message reflects the type of packet issued --- */
#define WMXPA_PV_CLEARREQ		(WM_APP+120)	/* Clear Request/Indication */
#define WMXPA_PV_RESETREQ		(WM_APP+121)	/* Reset Request/Indication */
#define WMXPA_PV_RESTARTREQ		(WM_APP+122)	/* Restart Request/Indication */
#define WMXPA_PV_DIAG			(WM_APP+123)	/* Diagnostic */


/* --- Timeout messages - a timeout occurred for a previously issued X.25 packet --- */
/* --- and appropriate recovery action has been undertaken. The mesage reflects  --- */
/* --- the type of packet originally transmitted ( and for which the timeout     --- */
/* --- occurred ).                                                               --- */
#define WMXPA_TO_RESTARTREQ		(WM_APP+130)	/* Restart Request/Indication */
#define WMXPA_TO_CALLREQ		(WM_APP+131)	/* Call Request */
#define WMXPA_TO_CLEARREQ		(WM_APP+132)	/* Clear Request/Indication */
#define WMXPA_TO_RESETREQ		(WM_APP+133)	/* Reset Request/Indication */
#define WMXPA_TO_INTERREQ		(WM_APP+134)	/* Interrupt */


/* --- Retry overflow messages - the retry limit was exceeded for a previously	 --- */
/* --- issued X.25 packet. The message reflects the type of the packet originally--- */
/* --- transmitted ( and for which the retry limit was exceeeded ).              --- */
#define WMXPA_RO_RESTARTREQ		(WM_APP+140)	/* Restart Request/Indication */
#define WMXPA_RO_CALLREQ		(WM_APP+141)	/* Call Request */
#define WMXPA_RO_CLEARREQ		(WM_APP+142)	/* Clear Request/Indication */
#define WMXPA_RO_RESETREQ		(WM_APP+143)	/* Reset Request/Indication */


/* --- Transmit buffers ready message - this will be received after an			 --- */
/* --- unsuccessful attempt to send some data with xpaWriteData(). If the call	 --- */
/* --- has failed due to transmit window being closed for the logical channel	 --- */
/* --- or because no tx buffers were available ( return code RC_NOT_READY ) this --- */
/* --- message will indicate that the applicaiton should retry sending data again--- */
/* --- since either tx window is now open or there's enough buffer space to      --- */
/* --- transmit the frame. Note that the lParam for this message is always 0.    --- */        
#define WMXPA_TXREADY			(WM_APP+150)


/* --- Data receive message - the received data frame sits inside data buffer  --- */
/* --- of XPA_MSG_DATA structure pointed by lParam                             --- */
#define WMXPA_RXDATA			(WM_APP+160)



/*
 * -----------------------------* Event Types   *-------------------------
 */

/* 
 * The following constants define different asynchronous event types. The event type
 * will be placed into EventType field of XPA_EVENT_DATA structure whenever the event
 * is fired by the API. You will notice that event type constants correspond directly
 * to WMXPA_ windows messages (they even have the same numeric value).
 */

/* --- Asynchronous X.25 packets reception events --- */

#define EVXPA_ASY_INCALL		(WM_APP+100)	/* Call Request/Incoming call */
#define EVXPA_ASY_CONNECTED		(WM_APP+101)	/* Call Accepted/Connected */

#define EVXPA_ASY_CLEARREQ		(WM_APP+102)	/* Clear Request/Indication */
#define EVXPA_ASY_CLEARCONF		(WM_APP+103)	/* Clear Confirmation */

#define EVXPA_ASY_RESETREQ		(WM_APP+104)	/* Reset Request/Indication */
#define EVXPA_ASY_RESETCONF		(WM_APP+105)	/* Reset Confirmation */

#define EVXPA_ASY_RESTARTREQ	(WM_APP+106)	/* Restart Request/Indication */
#define EVXPA_ASY_RESTARTCONF	(WM_APP+107)	/* Restart Confirmation */

#define EVXPA_ASY_INTERREQ		(WM_APP+108)	/* Interrupt */
#define EVXPA_ASY_INTERCONF		(WM_APP+109)	/* Interrupt Confirmation */

#define EVXPA_ASY_DTEREQ		(WM_APP+110)	/* DTE Registration Request */
#define EVXPA_ASY_DTECONF		(WM_APP+111)	/* DTE Registration Confirmation */

#define EVXPA_ASY_DIAG			(WM_APP+112)	/* Diagnostic message */


/* --- Protocol violation events  - a protocol violation occurred and an X.25 --- */
/* --- asynchronous packet has been automatically issued to notify the remote --- */
/* --- device of this violation. The event reflects the type of packet issued --- */
#define EVXPA_PV_CLEARREQ		(WM_APP+120)	/* Clear Request/Indication */
#define EVXPA_PV_RESETREQ		(WM_APP+121)	/* Reset Request/Indication */
#define EVXPA_PV_RESTARTREQ		(WM_APP+122)	/* Restart Request/Indication */
#define EVXPA_PV_DIAG			(WM_APP+123)	/* Diagnostic */


/* --- Timeout events - a timeout occurred for a previously issued X.25 packet  --- */
/* --- and appropriate recovery action has been undertaken. The mesage reflects --- */
/* --- the type of packet originally transmitted ( and for which the timeout    --- */
/* --- occurred ).                                                              --- */
#define EVXPA_TO_RESTARTREQ		(WM_APP+130)	/* Restart Request/Indication */
#define EVXPA_TO_CALLREQ		(WM_APP+131)	/* Call Request */
#define EVXPA_TO_CLEARREQ		(WM_APP+132)	/* Clear Request/Indication */
#define EVXPA_TO_RESETREQ		(WM_APP+133)	/* Reset Request/Indication */
#define EVXPA_TO_INTERREQ		(WM_APP+134)	/* Interrupt */


/* --- Retry overflow messages - the retry limit was exceeded for a previously   --- */
/* --- issued X.25 packet. The message reflects the type of the packet originally--- */
/* --- transmitted ( and for which the retry limit was exceeeded ).              --- */
#define EVXPA_RO_RESTARTREQ		(WM_APP+140)	/* Restart Request/Indication */
#define EVXPA_RO_CALLREQ		(WM_APP+141)	/* Call Request */
#define EVXPA_RO_CLEARREQ		(WM_APP+142)	/* Clear Request/Indication */
#define EVXPA_RO_RESETREQ		(WM_APP+143)	/* Reset Request/Indication */


/* --- Transmit buffers ready event - this will be received after an			 --- */
/* --- unsuccessful attempt to send some data with xpaWriteData(). If the call	 --- */
/* --- has failed due to transmit window being closed for the logical channel	 --- */
/* --- or because no tx buffers were available ( return code RC_NOT_READY ) this --- */
/* --- event will indicate that the applicaiton should retry sending data again  --- */
/* --- since either tx window is now open or there's enough buffer space to      --- */
/* --- transmit the frame. Note that there is no XPA_MSG_DATA for this event so  --- */
/* --- pMsgData inside XPA_EVENT_DATA will be set to NULL.						 --- */
#define EVXPA_TXREADY			(WM_APP+150)


/* --- A data receive event - the received data frame sits inside data buffer of --- */
/* --- XPA_MSG_DATA structure pointed by pMsgData inside XPA_EVENT_DATA structure--- */
#define EVXPA_RXDATA			(WM_APP+160)



/*
 * ----------------------------* Function prototypes  *------------------------
 */

#ifdef __cplusplus
extern "C" {			/* for C++ users */
#endif


/*
 * xpaOpen()	-	initializes XpipeAPI library when called forthe first time, opens XpipeAPI
 *					devices and connects to the specified card driver.
 *
 *	Arguments
 *					DeviceName  - [in] the name of the device that represents the card you
 *									   are going to work with, e.g. "XPIPE1".
 *										 This name can be viewed when you open the adapter 
 *									   properties from Control Panel/Network applet on NT4.
 *									   On Windows 2000/XP the first "Sangoma X.25 interface"
 *									   installed will be named "XPIPE1", second "XPIPE2" and so on.
 *									     If you set the DeviceName to NULL the API will pick up the
 *									   first XPIPE device it runs into, which may be exactly what
 *									   you want if you have only one S508 or S514 card installed.
 *
 *  Return values
 *					A positive integer greater representing the card number associated
 *					with the opened device if the operation was successfull, 0 otherwise. 
 *					Specific error number can be retrieved with xpaGetLastError().
 *
 *	Notes
 *					This should be a first call done by any process requiring XpipeAPI 
 *					services. The function sets up some internal structures and establishes
 *					communications with a given Xpipe card. Note that a process should call
 *					this function only once during its lifetime for the same DeviceName, any
 *					subsequent call will just increment the internal reference count. 
 *					The number of xpaOpen() calls per process for pareticular card should
 *					be balanced by the number of xpaClose() calls for this card.
 *
 *					The card number returned from this call is used to distinguish between
 *					cards in case one process requires services of more than one physical 
 *					card. 
 *
 *					Typically the process will call this function on start-up and call 
 *					xpaClose() on exit.
 *
 *	Remark
 *					In a current version a given process can work with one card only therefore 
 *					any xpaOpen() call other than first one will not have any effect. However
 *					if you call xpaClose() first and then xpaOpen() with a different device 
 *					it will allow you to use another card ( because its just like the first 
 *					call ever done from the process ).
 *
 *					The card number return in the current version will be always 1 ( or 0 in 
 *					case of any failure )
 */
int WINAPI
xpaOpen( LPCSTR DeviceName );


/*
 * xpaClose()	-	closes the device associated with the given CardNumber, cleans up the
 *					resources accossiated with the device, and cleans up the library if the
 *					reference count value shows it was the very last device used by the 
 *					process.
 *
 *	Arguments
 *					CardNumber	- [in] the card number ( returned by xpaOpen ) 
 *
 *  Return values
 *					none
 *
 *	Notes
 *					This should be a last call made by the process before exiting. The number 
 *					of xpaClose() calls for the given card should balance the number of xpaOpen().
 *					Typically the process will call xpaOpen() on start-up and call this function
 *					on exit.
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
void WINAPI
xpaClose( int CardNumber );


/*
 * xpaRegisterWindow() - this function registers a user window with the API to receive
 *						 notifications of various interesting events, such as X.25
 *						 asynchronous events, data reception and so on. The function
 *						 allows you to register a window for particular logical channel
 *						 on particular card in which case the window will receive notifications
 *						 of events that occured only on the channel of interest on the 
 *						 specified card, or you can register for all active logical channels
 *						 on particular card by specifying lcn 0, or you can register for
 *						 all logical channels on all opened cards by setting the CardNumber
 *						 to 0.
 *
 *	Arguments
 *					hNotifyWnd  - [in] handle of window that will receive notifications.
 *					CardNumber	- [in] the card number ( returned by xpaOpen ) 
 *					Lcn			- [in] logical channel of interest.
 *
 *  Return values
 *					TRUE is the window has been successfully registered. FALSE otherwise.
 *
 *	Notes
 *					Each window can be registered either for all logical channels or for 
 *					one particular logical channel. You can call xpaRegisterWindow() 
 *					multiple times for the same window - the most recently performed call 
 *					is the one that take effect. 
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
BOOL WINAPI
xpaRegisterWindow( HWND hNotifyWnd, int CardNumber, int Lcn );


/*
 * xpaDeregisterWindow() -  this function deregisters a user window from receiving XpipeAPI
 *							messages on particular card.
 *
 *	Arguments
 *					hNotifyWnd  - [in] handle of the previously registrered window.
 *					CardNumber	- [in] the card number ( returned by xpaOpen ) 
 *
 *  Return values
 *					none
 *
 *	Notes
 *					It is a good idea to deregister any windows that are not in use anymore
 *					or is about to be destroyed - this will definitely save some time ...
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
void WINAPI
xpaDeregisterWindow( HWND hNotifyWnd, int CardNumber );


/*
 * xpaMsgComplete() - this function should be called by the application message handler at the
 *					  end of processing of WMXPA_ messages. pMsgData should point to the message
 *					  data structure pointed by lParam of the original message.
 *
 *					  The function purpose is to make sure the memory allocated for the message
 *					  data will be released when nobody needs it anymore ....
 *
 *	Arguments
 *					pMsgData - [in] pointer to the original message data.
 *
 *  Return values
 *					none
 *
 *	Notes
 */
void WINAPI
xpaMsgComplete( XPA_MSG_DATA * pMsgData );


/*
 * xpaRegisterEvent() -  this function registers a windows event with the API. The event
 *						 will be set by the API whenever it needs to notify the user of
 *						 of various interesting events, such as X.25 asynchronous events,
 *						 data reception and so on. The function allows you to register an
 *						 event for particular logical channel on particular card in which
 *						 case it will be set to notify of events that occurred only on the
 *						 channel of interest, or you can register for all active logical
 *						 channels on particular card by specifying lcn 0, or you can register
 *						 for all logical channels on all opened cards by setting the 
 *						 CardNumber to 0.
 *						 
 *						 The third parameter is a pointer to XPA_EVENT_DATA structure allocated
 *						 by the user. This strucuture ( i.e. the memory allocated for it )
 *						 will be used as a container to pass the necessary information for
 *						 the event when its fired. When the event is fired the user will be
 *						 able to read off the event type, logical channel and other event data.
 *
 *	Arguments
 *					hNotifyEvt			- [in] handle of notification event.
 *					CardNumber			- [in] the card number ( returned by xpaOpen ) 
 *					Lcn					- [in] logical channel of interest.
 *					pEventDataContainer - [in] pointer to data container.
 *
 *  Return values
 *					TRUE  if the event has been successfully registered ( in terms of XpipeAPI ),
 *					FALSE otherwise, call xpaGetLastError() for specific error code.
 *
 *	Notes
 *					Any event window can be registered either for all logical channels or for 
 *					one particular logical channel. You can call xpaRegisterEvent() 
 *					multiple times for the same window - the most recently performed call 
 *					is the one that take effect.
 *				
 *					Please also make sure that the event should be in non-signalled state when
 *					passed to the API, and also no SetEvent() calls is performed from user
 *					application, because the API assumes it's the only one that sets it.
 *
 *					The function also initializes the user data container - so if there's 
 *					anything wrong with the storage FALSE will be returned and an error value
 *					of XPA_EINV_ARGS will be returned with xpaGetLastError();
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
BOOL WINAPI
xpaRegisterEvent( HANDLE hNotifyEvt, int CardNumber, int Lcn, XPA_EVENT_DATA * pEventDataContainer );


/*
 * xpaDeregisterEvent() -  this function deregisters a user event previously registred
 *						   with xpaRegisterEvent() for particular card number.
 *
 *	Arguments
 *					hNotifyEvent- [in] handle of previously registred notification event.
 *					CardNumber	- [in] the card number ( returned by xpaOpen ) 
 *
 *  Return values
 *					none
 *
 *	Notes
 *					It is a good idea to deregister events before exiting because its quite a lot
 *					of overhead for the API to keep it registered and not used.
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
void WINAPI
xpaDeregisterEvent( HANDLE hNotifyEvt, int CardNumber );


/*
 * xpaEventComplete() - this function should be called by the application event handler once its 
 *						done processing the event previously fired by the API. It doesn't matter
 *						whether the event is reset or not - this function will reset it anyways.
 *
 *						This is the only way for API to know that the user application has
 *						processed the previously fired event, so it can fire another one if there's
 *						any queued. In addition the function makes sure the memory allocated for the
 *						message data will be released when nobody needs it anymore ....
 *
 *	Arguments
 *					hNotifyEvt - [in] handle to event registreed with the API and being "acknowledged"
 *
 *  Return values
 *					none
 *
 *	Notes
 */
void WINAPI
xpaEventComplete( HANDLE hNotifyEvt );


/*
 * xpaGetLastError() - returns the code of the most recently occurred error. 
 *
 *	Arguments
 *					none
 *
 *  Return values
 *					the code of the most recently occurred error.
 *
 *	Notes
 *					In current version this value is shared between multiple threads of 
 *					the same process so do not trust it to control your application flow
 *					- use it for debugging purposes only.
 */
int WINAPI
xpaGetLastError( void );


/*
 * xpaReadConfiguration() - returns the X.25/HDLC configuration data for specified card.
 *
 *	Arguments
 *					CardNumber	- [in] the card number ( returned by xpaOpen ) 
 *					pX25cfg		- [in/out] pointer to configuration structure which will
 *									       be filled in on successful completion.
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaReadConfiguration( int CardNumber, XPA_X25CFG *pX25cfg );


/*
 * xpaPlaceCall() - issues X.25 Call Request/Incoming Call packet on the next available
 *					SVC on specified card.
 *
 *	Arguments
 *					CardNumber- [in] the card number ( returned by xpaOpen ) 
 *
 *					pLcn	  - [out] logical channel on which the Call Request was 
 *									 issued.
 *
 *					CallData  - [in]  a null-terminated ASCII string containing the call 
 *									 data in the following format :
 *								    
 *									 -d[called DTE address] -s[calling DTE address]
 *									 -f[facilities] -u[user data]
 *
 *									 e.g. -d8120012 -f0101 -uC2
 *
 *					QDMbits		- [in]  the status of Q, D and M bits ( bits 2,1,0 respectively )
 *										in the data packet to be transmitted.
 *
 *									
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *					the returned LCN ( logical channel number ) is valid only when 
 *					0 is returned from this function.
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaPlaceCall( int CardNumber, int * pLcn, LPSTR CallData, BYTE QDMbits );


/*
 * xpaAcceptCall() - issues X.25 Call Accepted/Connected packet on specified card
 *					 in response to a Call Request/Incoming Call previously received
 *					 ( i.e. in response to WMXPA_ASY_INCALL message ).
 *
 *	Arguments
 *					CardNumber- [in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn		  - [in]  logical channel on which the Call Accepted packet
 *									 should be issued.
 *
 *					CallData  - [in]  a null-terminated ASCII string containing the call 
 *									 data in the same format as for xpaPlaceCall() command.
 *									 Can be set to NULL.
 *
 *					QDMbits	  - [in]  the status of Q, D and M bits ( bits 2,1,0 respectively )
 *									 in the Call Accepted packet ( in response to the bits
 *									 being set in received Call Request packet ).
 *
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaAcceptCall( int CardNumber, int Lcn, LPSTR CallData, BYTE QDMbits );


/*
 * xpaClearCall()  - issues X.25 Clear Request/Indication packet on selected logical
 *					 channel on specified card.
 *
 *	Arguments
 *					CardNumber- [in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn		  - [in]  logical channel on which the Clear Request packet
 *									 should be issued.
 *
 *					ClearData - [in] a null-terminated ASCII string containing the clear
 *									 argument data. Can be set to NULL.
 *
 *					Flush	  - [in]  if TRUE then all the receive and transmit buffers
 *									 ( for the Lcn ) will be flushed before the Clear 
 *									 packet is issued.
 *
 *									 if FALSE then the Clear packet will not be issued
 *									 if there are any queued data buffers on this logical
 *									 channel, instead a return code of RC_CHANNEL_IN_USE
 *									 will be passed to the application.
 *
 *					Cause	  - [in]  X.25 cause field.
 *
 *					Diagn	  - [in]	 X.25 diagnostic field.
 *									 
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaClearCall( int CardNumber, int Lcn, LPSTR ClearData, BOOL Flush, BYTE Cause, BYTE Diagn );


/*
 * xpaReset()      - issues X.25 Reset Request/Indication packet on selected logical
 *					 channel on specified card.
 *
 *	Arguments
 *
 *					CardNumber- [in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn		  - [in]  logical channel on which the Reset Request packet
 *									 should be issued.
 *
 *					Flush	  - [in]  if TRUE then all the receive and transmit buffers
 *									 ( for the Lcn ) will be flushed before the Reset 
 *									 packet is issued.
 *
 *									 if FALSE then the Reset packet will not be issued
 *									 if there are any queued data buffers on this logical
 *									 channel, instead a return code of RC_CHANNEL_IN_USE
 *									 will be passed to the application.
 *
 *					Cause	  - [in]  X.25 cause field.
 *
 *					Diagn	  - [in]	 X.25 diagnostic field.
 *									 
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaReset( int CardNumber, int Lcn, BOOL Flush, BYTE Cause, BYTE Diagn );


/*
 * xpaInterrupt()  - issues X.25 Interrupt packet on selected logical channel on 
 *					 specified card.
 *
 *	Arguments
 *
 *					CardNumber - [in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn		   - [in]  logical channel on which the Interrupt packet
 *									 should be issued.
 *
 *					IntrData   - [in]  user data to be transmitted with Interrupt.
 *
 *					IntrDataLen- [in] length of the above. The maximum data length is 32
 *									  bytes if configured to support CCITT 1984/88, or
 *									  1 byte if CCITT 1980 compatibility is required.
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaInterrupt( int CardNumber, int Lcn, BYTE *IntrData, int IntrDataLen );


/*
 * xpaRegistration()  - issues X.25 DTE Registration packet on specified card.
 *
 *	Arguments
 *
 *					CardNumber- [in] the card number ( returned by xpaOpen ) 
 *
 *					RegData   - [in] a null-terminated ASCII string containing the registration
 *								   data. Can be set to NULL.
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaRegistration( int CardNumber, LPSTR RegData );


/*
 * xpaConfirmRegistration()  - issues X.25 DCE Registration Confirmation packet on specified
 *							   card.
 *
 *	Arguments
 *
 *					CardNumber- [in] the card number ( returned by xpaOpen ) 
 *
 *					RegData   - [in] a null-terminated ASCII string containing the registration
 *								   data. Can be set to NULL.
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaConfirmRegistration( int CardNumber, LPSTR RegData );


/*
 * xpaGetLcnStatus() - returns transmit/receive information for selected logical channel
 *					   on specified card.
 *
 *	Arguments
 *
 *					CardNumber-[in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn		 - [in]  logical channel of interest.
 *
 *					IsDTM	 - [out] TRUE if the selected logical channel is in data 
 *									 transfer mode, FALSE otherwise.
 *					
 *					TxWinOpen- [out] TRUE if the transmit window is open and data packets 
 *									 may be issued for this logical channel by using 
 *									 xpaWriteData() command.
 *
 *					InPackets- [out] number of incoming data packets queued for this logical
 *									 channel and available for reception by the application.
 *
 *  Return values
 *					0 if the command has been executed successfully and the data of interest
 *					is available at locations pointed by IsDTM, TxWinOpen and InPackets, or 
 *					-1 - if we were unable to execute the command for some reason, call 
 *					xpaGetLastError() for specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaGetLcnStatus( int CardNumber, int Lcn, BOOL *IsDTM, BOOL *TxWinOpen, int *InPackets );


/*
 * xpaWriteData() - sends user data to the card for onward transmission to the network on
 *					selected logical channel on specified card.
 *
 *	Arguments
 *
 *					CardNumber  - [in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn			- [in]  logical channel to send the data on.
 *
 *					WriteData	- [in]  buffer containing user data to send.
 *					
 *					WriteDataLen- [in]  length of the data inside the above buffer.
 *
 *					QDMbits		- [in]  the status of Q, D and M bits ( bits 2,1,0 respectively )
 *										in the data packet to be transmitted.
 *
 *					The function attempts to send the data using XPA_X25_WRITE command. If the
 *					return value indicates that the data was not queued due to buffering or
 *					transmit window problems ( return code RC_NOT_READY ( 0x33 )) the driver
 *					will set transmit interrupt for the selected logical channel and the 
 *					application will be notified with WMXPA_TXREADY message whenever data 
 *					transmission is possible on selected logical channel.
 *
 *					The maximum data length is 1024 bytes or less depending on the protocol 
 *					configuration settings ( see xpaReadConfiguration() command ).
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaWriteData( int CardNumber, int Lcn, BYTE *WriteData, int WriteDataLen, BYTE QDMbits );


/*
 * xpaReadData() -  receives an X.25 data packet from the network on selected logical channel.
 *
 *	Arguments
 *
 *					CardNumber  - [in] the card number ( returned by xpaOpen ) 
 *
 *					Lcn			- [in]  logical channel to receive the data on.
 *
 *					ReadData	- [in/out]  buffer to place the received data in.
 *					
 *					ReadDataLen- [out]  length of the received data inside the above buffer.
 *
 *					QDMbits		- [out]  the status of Q, D and M bits ( bits 2,1,0 respectively )
 *										 in the data packet received.
 *
 *					The function attempts to receive the data using XPA_X25_READ command. Note 
 *					that the ReadData buffer must be at least as large as the configured maximum
 *					data frame size.
 *
 *  Return values
 *					board level return code if the command has been executed 
 *					successfully, or -1 - if we were unable to execute the 
 *					command for some reason, call xpaGetLastError() for 
 *					specific error code.
 *
 *	Notes
 *					Please note that although it is a totally legitimate way to receive data
 *					from the board the preferred way is to handle WMXPA_RXDATA messages since 
 *					this way you will avoid busy loops while waiting for the data to arrive and
 *					also you won't have to allocate extra space for the data buffer ( it has
 *					to be at least as large as the maximum data frame, remember ? )
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaReadData( int CardNumber, int Lcn, BYTE *ReadData, int *ReadDataLen, BYTE *QDMbits );


/*
 * xpaDoCommand() - executes on-board command "as is".
 *
 *	Arguments
 *
 *					CardNumber  - [in] the card number ( returned by xpaOpen ) 
 *
 *					pcBlock		- [in/out] pointer to control block which will be passed
 *										   to the board, executed and filled in with 
 *										   the result.
 *  Return values
 *					board level return code ( copy of the pcBlock->return_code ) if
 *					the command has been executed successfully, or -1 - if we were 
 *					unable to execute the command for some reason, call xpaGetLastError()
 *					for specific error code.
 *
 *	Notes
 *					the function does not check the validity of commands - i.e.
 *					it just tries to pass it to the board and execute it there,
 *					therefore you've got total control over the board with this
 *					function - well, i hope you know what you're doing ....
 *
 *	Remark
 *					Current version of the API supports only one card per process therefore
 *					the CardNumber parameter is ignored.
 */
int WINAPI
xpaDoCommand( int CardNumber, PXPA_CBLOCK pcBlock );


#ifdef __cplusplus
}
#endif

#endif /* _XPIPEAPI_H_ */